@@ -14,32 +14,31 @@
 
 """Fake implementations for different file objects."""
 
+from __future__ import annotations
+
 import errno
 import io
 import os
 import sys
 import traceback
+import weakref
 from stat import (
     S_IFREG,
     S_IFDIR,
 )
 from types import TracebackType
 from typing import (
-    List,
-    Optional,
-    Callable,
     Union,
     Any,
-    Dict,
     cast,
     AnyStr,
     NoReturn,
-    Iterator,
     TextIO,
-    Type,
     TYPE_CHECKING,
 )
 
+from collections.abc import Callable, Iterator
+
 from pyfakefs import helpers
 from pyfakefs.helpers import (
     FakeStatResult,
@@ -135,12 +134,12 @@ class FakeFile:
         self,
         name: AnyStr,
         st_mode: int = S_IFREG | helpers.PERM_DEF_FILE,
-        contents: Optional[AnyStr] = None,
-        filesystem: Optional["FakeFilesystem"] = None,
-        encoding: Optional[str] = None,
-        errors: Optional[str] = None,
-        side_effect: Optional[Callable[["FakeFile"], None]] = None,
-        open_modes: Optional[_OpenModes] = None,
+        contents: AnyStr | None = None,
+        filesystem: FakeFilesystem | None = None,
+        encoding: str | None = None,
+        errors: str | None = None,
+        side_effect: Callable[[FakeFile], None] | None = None,
+        open_modes: _OpenModes | None = None,
     ):
         """
         Args:
@@ -164,8 +163,10 @@ class FakeFile:
         # to be backwards compatible regarding argument order, we raise on None
         if filesystem is None:
             raise ValueError("filesystem shall not be None")
-        self.filesystem: "FakeFilesystem" = filesystem
-        self._side_effect: Optional[Callable] = side_effect
+        self._filesystem: weakref.ReferenceType[FakeFilesystem] = weakref.ref(
+            filesystem
+        )
+        self._side_effect: Callable | None = side_effect
         self.name: AnyStr = name  # type: ignore[assignment]
         self.stat_result = FakeStatResult(
             filesystem.is_windows_fs,
@@ -177,26 +178,48 @@ class FakeFile:
             st_mode |= S_IFREG
         self.stat_result.st_mode = st_mode
         self.st_size: int = 0
-        self.encoding: Optional[str] = real_encoding(encoding)
+        self.encoding: str | None = real_encoding(encoding)
         self.errors: str = errors or "strict"
-        self._byte_contents: Optional[bytes] = self._encode_contents(contents)
+        self._byte_contents: bytes | None = self._encode_contents(contents)
         self.stat_result.st_size = (
             len(self._byte_contents) if self._byte_contents is not None else 0
         )
         self.epoch: int = 0
-        self.parent_dir: Optional[FakeDirectory] = None
+        self.parent_dir: weakref.ReferenceType | None = None
         # Linux specific: extended file system attributes
-        self.xattr: Dict = {}
+        self.xattr: dict = {}
         self.opened_as: AnyString = ""
         self.open_modes = open_modes
 
+    def __getstate__(self):
+        """Handle weakref to allow pickling of the filesystem and parent dir"""
+        s = self.__dict__.copy()
+        s["_filesystem"] = s["_filesystem"]()
+        parent = s["parent_dir"]
+        s["parent_dir"] = parent() if parent is not None else None
+        return s
+
+    def __setstate__(self, state):
+        self.__dict__ = state.copy()
+        self.__dict__["_filesystem"] = weakref.ref(self.__dict__["_filesystem"])
+        parent = self.__dict__["parent_dir"]
+        self.__dict__["parent_dir"] = (
+            weakref.ref(parent) if parent is not None else None
+        )
+
     @property
-    def byte_contents(self) -> Optional[bytes]:
+    def filesystem(self) -> FakeFilesystem:
+        fs = self._filesystem()
+        assert fs is not None
+        return fs
+
+    @property
+    def byte_contents(self) -> bytes | None:
         """Return the contents as raw byte array."""
         return self._byte_contents
 
     @property
-    def contents(self) -> Optional[str]:
+    def contents(self) -> str | None:
         """Return the contents as string with the original encoding."""
         if isinstance(self.byte_contents, bytes):
             return self.byte_contents.decode(
@@ -236,7 +259,7 @@ class FakeFile:
         self.stat_result.st_mtime = val
 
     def set_large_file_size(self, st_size: int) -> None:
-        """Sets the self.st_size attribute and replaces self.content with None.
+        """Sets the self.st_size attribute and replaces self.content with `None`.
 
         Provided specifically to simulate very large files without regards
         to their content (which wouldn't fit in memory).
@@ -247,19 +270,19 @@ class FakeFile:
           st_size: (int) The desired file size
 
         Raises:
-          OSError: if the st_size is not a non-negative integer,
-                   or if st_size exceeds the available file system space
+          OSError: if ``st_size`` is not a non-negative integer,
+                   or if ``st_size`` exceeds the available file system space
         """
         self._check_positive_int(st_size)
         if self.st_size:
             self.size = 0
-        if self.filesystem:
+        if self._filesystem():
             self.filesystem.change_disk_usage(st_size, self.name, self.st_dev)
         self.st_size = st_size
         self._byte_contents = None
 
     def _check_positive_int(self, size: int) -> None:
-        # the size should be an positive integer value
+        # the size should be a positive integer value
         if not is_int_type(size) or size < 0:
             self.filesystem.raise_os_error(errno.ENOSPC, self.name)
 
@@ -269,7 +292,7 @@ class FakeFile:
         """
         return self._byte_contents is None
 
-    def _encode_contents(self, contents: Union[str, bytes, None]) -> Optional[bytes]:
+    def _encode_contents(self, contents: str | bytes | None) -> bytes | None:
         if is_unicode_string(contents):
             contents = bytes(
                 cast(str, contents),
@@ -286,11 +309,11 @@ class FakeFile:
             contents: string, new content of file.
 
         Returns:
-            True if the contents have been changed.
+            `True` if the contents have been changed.
 
         Raises:
-              OSError: if the st_size is not a non-negative integer,
-                   or if st_size exceeds the available file system space
+              OSError: if the `st_size` is not a non-negative integer,
+                   or if `st_size` exceeds the available file system space
         """
         byte_contents = self._encode_contents(contents)
         changed = self._byte_contents != byte_contents
@@ -305,18 +328,18 @@ class FakeFile:
         self.epoch += 1
         return changed
 
-    def set_contents(self, contents: AnyStr, encoding: Optional[str] = None) -> bool:
+    def set_contents(self, contents: AnyStr, encoding: str | None = None) -> bool:
         """Sets the file contents and size and increases the modification time.
         Also executes the side_effects if available.
 
         Args:
           contents: (str, bytes) new content of file.
           encoding: (str) the encoding to be used for writing the contents
-                    if they are a unicode string.
+                    if they are a Unicode string.
                     If not given, the locale preferred encoding is used.
 
         Returns:
-            True if the contents have been changed.
+            `True` if the contents have been changed.
 
         Raises:
           OSError: if `st_size` is not a non-negative integer,
@@ -362,11 +385,11 @@ class FakeFile:
     @property
     def path(self) -> AnyStr:  # type: ignore[type-var]
         """Return the full path of the current object."""
-        names: List[AnyStr] = []  # pytype: disable=invalid-annotation
-        obj: Optional[FakeFile] = self
+        names: list[AnyStr] = []  # pytype: disable=invalid-annotation
+        obj: FakeFile | None = self
         while obj:
             names.insert(0, matching_string(self.name, obj.name))  # type: ignore
-            obj = obj.parent_dir
+            obj = obj.parent_dir() if obj.parent_dir else None
         sep = self.filesystem.get_path_separator(names[0])
         if names[0] == sep:
             names.pop(0)
@@ -408,7 +431,7 @@ class FakeFile:
             permission_bits: The permission bits as set for the user.
 
         Returns:
-            True if the permissions are set in the correct class (user/group/other).
+            `True` if the permissions are set in the correct class (user/group/other).
         """
         if helpers.get_uid() == self.stat_result.st_uid:
             return self.st_mode & permission_bits == permission_bits
@@ -418,7 +441,7 @@ class FakeFile:
 
 
 class FakeNullFile(FakeFile):
-    def __init__(self, filesystem: "FakeFilesystem") -> None:
+    def __init__(self, filesystem: FakeFilesystem) -> None:
         super().__init__(filesystem.devnull, filesystem=filesystem, contents="")
 
     @property
@@ -438,8 +461,8 @@ class FakeFileFromRealFile(FakeFile):
     def __init__(
         self,
         file_path: str,
-        filesystem: "FakeFilesystem",
-        side_effect: Optional[Callable] = None,
+        filesystem: FakeFilesystem,
+        side_effect: Callable | None = None,
     ) -> None:
         """
         Args:
@@ -458,7 +481,7 @@ class FakeFileFromRealFile(FakeFile):
         self.contents_read = False
 
     @property
-    def byte_contents(self) -> Optional[bytes]:
+    def byte_contents(self) -> bytes | None:
         if not self.contents_read:
             self.contents_read = True
             with io_open(self.file_path, "rb") as f:
@@ -483,7 +506,7 @@ class FakeDirectory(FakeFile):
         self,
         name: str,
         perm_bits: int = helpers.PERM_DEF,
-        filesystem: Optional["FakeFilesystem"] = None,
+        filesystem: FakeFilesystem | None = None,
     ):
         """
         Args:
@@ -496,18 +519,18 @@ class FakeDirectory(FakeFile):
         # directories have the link count of contained entries,
         # including '.' and '..'
         self.st_nlink += 1
-        self._entries: Dict[str, AnyFile] = {}
+        self._entries: dict[str, AnyFile] = {}
 
-    def set_contents(self, contents: AnyStr, encoding: Optional[str] = None) -> bool:
+    def set_contents(self, contents: AnyStr, encoding: str | None = None) -> bool:
         raise self.filesystem.raise_os_error(errno.EISDIR, self.path)
 
     @property
-    def entries(self) -> Dict[str, FakeFile]:
+    def entries(self) -> dict[str, FakeFile]:
         """Return the list of contained directory entries."""
         return self._entries
 
     @property
-    def ordered_dirs(self) -> List[str]:
+    def ordered_dirs(self) -> list[str]:
         """Return the list of contained directory entry names ordered by
         creation order.
         """
@@ -531,14 +554,14 @@ class FakeDirectory(FakeFile):
             and not self.filesystem.is_windows_fs
             and not self.has_permission(helpers.PERM_WRITE)
         ):
-            raise OSError(errno.EACCES, "Permission Denied", self.path)
+            self.filesystem.raise_os_error(errno.EACCES, self.path)
 
         path_object_name: str = to_string(path_object.name)
         if path_object_name in self.entries:
             self.filesystem.raise_os_error(errno.EEXIST, self.path)
 
         self._entries[path_object_name] = path_object
-        path_object.parent_dir = self
+        path_object.parent_dir = weakref.ref(self)
         if path_object.st_ino is None:
             self.filesystem.last_ino += 1
             path_object.st_ino = self.filesystem.last_ino
@@ -579,7 +602,7 @@ class FakeDirectory(FakeFile):
 
         Args:
             pathname_name: Basename of the child object to remove.
-            recursive: If True (default), the entries in contained directories
+            recursive: If `True` (default), the entries in contained directories
                 are deleted first. Used to propagate removal errors
                 (e.g. permission problems) from contained entries.
 
@@ -600,16 +623,8 @@ class FakeDirectory(FakeFile):
                     # tempfile uses unlink based on the real OS while deleting
                     # a temporary file, so we ignore that error in this specific case
                     st = traceback.extract_stack(limit=6)
-                    if sys.version_info < (3, 10):
-                        if (
-                            st[0].name == "TemporaryFile"
-                            and st[0].line == "_os.unlink(name)"
-                        ):
-                            raise_error = False
-                    else:
-                        # TemporaryFile implementation has changed in Python 3.10
-                        if st[0].name == "opener" and st[0].line == "_os.unlink(name)":
-                            raise_error = False
+                    if st[0].name == "opener" and st[0].line == "_os.unlink(name)":
+                        raise_error = False
                 if raise_error:
                     self.filesystem.raise_os_error(errno.EACCES, pathname_name)
         else:
@@ -642,14 +657,14 @@ class FakeDirectory(FakeFile):
         """Setting the size is an error for a directory."""
         raise self.filesystem.raise_os_error(errno.EISDIR, self.path)
 
-    def has_parent_object(self, dir_object: "FakeDirectory") -> bool:
+    def has_parent_object(self, dir_object: FakeDirectory) -> bool:
         """Return `True` if dir_object is a direct or indirect parent
         directory, or if both are the same object."""
-        obj: Optional[FakeDirectory] = self
+        obj: FakeDirectory | None = self
         while obj:
             if obj == dir_object:
                 return True
-            obj = obj.parent_dir
+            obj = obj.parent_dir() if obj.parent_dir else None
         return False
 
     def __str__(self) -> str:
@@ -671,9 +686,9 @@ class FakeDirectoryFromRealDirectory(FakeDirectory):
     def __init__(
         self,
         source_path: AnyPath,
-        filesystem: "FakeFilesystem",
+        filesystem: FakeFilesystem,
         read_only: bool,
-        target_path: Optional[AnyPath] = None,
+        target_path: AnyPath | None = None,
     ):
         """
         Args:
@@ -707,7 +722,7 @@ class FakeDirectoryFromRealDirectory(FakeDirectory):
         self.contents_read = False
 
     @property
-    def entries(self) -> Dict[str, FakeFile]:
+    def entries(self) -> dict[str, FakeFile]:
         """Return the list of contained directory entries, loading them
         if not already loaded."""
         if not self.contents_read:
@@ -751,16 +766,15 @@ class FakeFileWrapper:
         self,
         file_object: FakeFile,
         file_path: AnyStr,
-        update: bool,
-        read: bool,
-        append: bool,
+        open_modes: _OpenModes,
+        allow_update: bool,
         delete_on_close: bool,
-        filesystem: "FakeFilesystem",
-        newline: Optional[str],
+        filesystem: FakeFilesystem,
+        newline: str | None,
         binary: bool,
         closefd: bool,
-        encoding: Optional[str],
-        errors: Optional[str],
+        encoding: str | None,
+        errors: str | None,
         buffering: int,
         raw_io: bool,
         opened_as_fd: bool,
@@ -768,9 +782,8 @@ class FakeFileWrapper:
     ):
         self.file_object = file_object
         self.file_path = file_path  # type: ignore[var-annotated]
-        self._append = append
-        self._read = read
-        self.allow_update = update
+        self.open_modes = open_modes
+        self.allow_update = allow_update
         self._closefd = closefd
         self._file_epoch = file_object.epoch
         self.raw_io = raw_io
@@ -789,7 +802,7 @@ class FakeFileWrapper:
         contents = file_object.byte_contents
         self._encoding = encoding or get_locale_encoding()
         errors = errors or "strict"
-        self._io: Union[BinaryBufferIO, TextBufferIO] = (
+        self._io: BinaryBufferIO | TextBufferIO = (
             BinaryBufferIO(contents)
             if binary
             else TextBufferIO(
@@ -801,8 +814,8 @@ class FakeFileWrapper:
         self._flush_pos = 0
         if contents:
             self._flush_pos = len(contents)
-            if update:
-                if not append:
+            if self.allow_update:
+                if not self.open_modes.append:
                     self._io.seek(0)
                 else:
                     self._io.seek(self._flush_pos)
@@ -810,29 +823,35 @@ class FakeFileWrapper:
 
         if delete_on_close:
             assert filesystem, "delete_on_close=True requires filesystem"
-        self._filesystem = filesystem
+        self._filesystem = weakref.ref(filesystem)
         self.delete_on_close = delete_on_close
         # override, don't modify FakeFile.name, as FakeFilesystem expects
         # it to be the file name only, no directories.
         self.name = file_object.opened_as
-        self.filedes: Optional[int] = None
+        self.filedes: int | None = None
 
-    def __enter__(self) -> "FakeFileWrapper":
+    @property
+    def filesystem(self) -> FakeFilesystem:
+        fs = self._filesystem()
+        assert fs is not None
+        return fs
+
+    def __enter__(self) -> FakeFileWrapper:
         """To support usage of this fake file with the 'with' statement."""
         return self
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         """To support usage of this fake file with the 'with' statement."""
         self.close()
 
     def _raise(self, message: str) -> NoReturn:
         if self.raw_io:
-            self._filesystem.raise_os_error(errno.EBADF, self.file_path)
+            self.filesystem.raise_os_error(errno.EBADF, self.file_path)
         raise io.UnsupportedOperation(message)
 
     def get_object(self) -> FakeFile:
@@ -845,13 +864,13 @@ class FakeFileWrapper:
         """Return the file descriptor of the file object."""
         if self.filedes is not None:
             return self.filedes
-        raise OSError(errno.EBADF, "Invalid file descriptor")
+        self.filesystem.raise_os_error(errno.EBADF)
 
     def close(self) -> None:
         """Close the file."""
         self.close_fd(self.filedes)
 
-    def close_fd(self, fd: Optional[int]) -> None:
+    def close_fd(self, fd: int | None) -> None:
         """Close the file for the given file descriptor."""
 
         # ignore closing a closed file
@@ -867,21 +886,21 @@ class FakeFileWrapper:
                     # if we get here, we have an open file descriptor
                     # without write permission, which has to be closed
                     assert self.filedes
-                    self._filesystem.close_open_file(self.filedes)
+                    self.filesystem.close_open_file(self.filedes)
                 raise
 
-            if self._filesystem.is_windows_fs and self._changed:
+            if self.filesystem.is_windows_fs and self._changed:
                 self.file_object.st_mtime = helpers.now()
 
         assert fd is not None
         if self._closefd:
-            self._filesystem.close_open_file(fd)
+            self.filesystem.close_open_file(fd)
         else:
-            open_files = self._filesystem.open_files[fd]
+            open_files = self.filesystem.open_files[fd]
             assert open_files is not None
             open_files.remove(self)
         if self.delete_on_close:
-            self._filesystem.remove_object(
+            self.filesystem.remove_object(
                 self.get_object().path  # type: ignore[arg-type]
             )
 
@@ -890,6 +909,27 @@ class FakeFileWrapper:
         """Simulate the `closed` attribute on file."""
         return not self._is_open()
 
+    @property
+    def mode(self) -> str:
+        if self.open_modes.append:
+            m = "ab" if self._binary else "a"
+            return m + "+" if self.open_modes.can_read else m
+        if self.open_modes.truncate:
+            if self._binary:
+                if (
+                    sys.version_info >= (3, 15)
+                    and self.open_modes.can_write
+                    and self.open_modes.can_read
+                ):
+                    return "wb+"
+                return "rb+" if self.open_modes.can_read else "wb"
+            return "w+" if self.open_modes.can_read else "w"
+        if self.open_modes.must_not_exist:
+            m = "xb" if self._binary else "x"
+            return m + "+" if self.open_modes.can_read else m
+        m = "rb" if self._binary else "r"
+        return m + "+" if self.open_modes.can_write else m
+
     def _try_flush(self, old_pos: int) -> None:
         """Try to flush and reset the position if it fails."""
         flush_pos = self._flush_pos
@@ -910,7 +950,7 @@ class FakeFileWrapper:
         self._check_open_file()
 
         if self.allow_update:
-            if self._append:
+            if self.open_modes.append:
                 contents = self._io.getvalue()
                 self._sync_io()
                 old_contents = self.file_object.byte_contents
@@ -923,7 +963,7 @@ class FakeFileWrapper:
             changed = self.file_object.set_contents(contents, self._encoding)
             self.update_flush_pos()
             if changed:
-                if self._filesystem.is_windows_fs:
+                if self.filesystem.is_windows_fs:
                     self._changed = True
                 else:
                     current_time = helpers.now()
@@ -938,27 +978,27 @@ class FakeFileWrapper:
                 content_length = len(self.file_object.byte_contents)
             # an error is only raised if there is something to flush
             if content_length != buf_length:
-                self._filesystem.raise_os_error(errno.EBADF)
+                self.filesystem.raise_os_error(errno.EBADF)
 
     def update_flush_pos(self) -> None:
         self._flush_pos = self._io.tell()
 
     def _flush_related_files(self) -> None:
-        for open_files in self._filesystem.open_files[3:]:
+        for open_files in self.filesystem.open_files[3:]:
             if open_files is not None:
                 for open_file in open_files:
                     if (
                         open_file is not self
                         and isinstance(open_file, FakeFileWrapper)
                         and self.file_object == open_file.file_object
-                        and not open_file._append
+                        and not open_file.open_modes.append
                     ):
                         open_file._sync_io()
 
     def seek(self, offset: int, whence: int = 0) -> None:
         """Move read/write pointer in 'file'."""
         self._check_open_file()
-        if not self._append:
+        if not self.open_modes.append:
             self._io.seek(offset, whence)
         else:
             self._read_seek = offset
@@ -976,7 +1016,7 @@ class FakeFileWrapper:
         if not self.is_stream:
             self.flush()
 
-        if not self._append:
+        if not self.open_modes.append:
             return self._io.tell()
         if self._read_whence:
             write_seek = self._io.tell()
@@ -986,6 +1026,14 @@ class FakeFileWrapper:
             self._io.seek(write_seek)
         return self._read_seek
 
+    def readable(self) -> bool:
+        """Returns the readable state of the file."""
+        return self.open_modes.can_read
+
+    def writable(self) -> bool:
+        """Returns the writable state of the file."""
+        return self.open_modes.can_write
+
     def _sync_io(self) -> None:
         """Update the stream with changes to the file object contents."""
         if self._file_epoch == self.file_object.epoch:
@@ -1001,7 +1049,7 @@ class FakeFileWrapper:
         self._io.seek(0)
         self._io.truncate()
         self._io.putvalue(contents)
-        if not self._append:
+        if not self.open_modes.append:
             self._io.seek(whence)
 
     def _read_wrappers(self, name: str) -> Callable:
@@ -1117,7 +1165,7 @@ class FakeFileWrapper:
                 self._try_flush(old_pos)
                 if not flush_all:
                     ret_value = io_attr(*args, **kwargs)
-            if self._append:
+            if self.open_modes.append:
                 self._read_seek = self._io.tell()
                 self._read_whence = 0
             return ret_value
@@ -1125,14 +1173,14 @@ class FakeFileWrapper:
         return write_wrapper
 
     def _adapt_size_for_related_files(self, size: int) -> None:
-        for open_files in self._filesystem.open_files[3:]:
+        for open_files in self.filesystem.open_files[3:]:
             if open_files is not None:
                 for open_file in open_files:
                     if (
                         open_file is not self
                         and isinstance(open_file, FakeFileWrapper)
                         and self.file_object == open_file.file_object
-                        and cast(FakeFileWrapper, open_file)._append
+                        and cast(FakeFileWrapper, open_file).open_modes.append
                     ):
                         open_file._read_seek += size
 
@@ -1146,7 +1194,7 @@ class FakeFileWrapper:
 
         def truncate_wrapper(*args, **kwargs):
             """Wrap truncate call to call flush after truncate."""
-            if self._append:
+            if self.open_modes.append:
                 self._io.seek(self._read_seek, self._read_whence)
             size = io_attr(*args, **kwargs)
             self.flush()
@@ -1165,6 +1213,7 @@ class FakeFileWrapper:
 
         return truncate_wrapper
 
+    @property
     def size(self) -> int:
         """Return the content size in bytes of the wrapped file."""
         return self.file_object.st_size
@@ -1179,7 +1228,7 @@ class FakeFileWrapper:
 
         if reading or writing:
             self._check_open_file()
-        if not self._read and reading:
+        if not self.readable() and reading:
             return self._read_error()
         if not self.opened_as_fd and not self.allow_update and writing:
             return self._write_error()
@@ -1188,11 +1237,11 @@ class FakeFileWrapper:
             self._sync_io()
             if not self.is_stream:
                 self.flush()
-            if not self._filesystem.is_windows_fs:
+            if not self.filesystem.is_windows_fs:
                 self.file_object.st_atime = helpers.now()
         if truncate:
             return self._truncate_wrapper()
-        if self._append:
+        if self.open_modes.append:
             if reading:
                 return self._read_wrappers(name)
             elif not writing:
@@ -1206,7 +1255,7 @@ class FakeFileWrapper:
         def read_error(*args, **kwargs):
             """Throw an error unless the argument is zero."""
             if args and args[0] == 0:
-                if self._filesystem.is_windows_fs and self.raw_io:
+                if self.filesystem.is_windows_fs and self.raw_io:
                     return b"" if self._binary else ""
             self._raise("File is not open for reading.")
 
@@ -1216,15 +1265,15 @@ class FakeFileWrapper:
         def write_error(*args, **kwargs):
             """Throw an error."""
             if self.raw_io:
-                if self._filesystem.is_windows_fs and args and len(args[0]) == 0:
+                if self.filesystem.is_windows_fs and args and len(args[0]) == 0:
                     return 0
             self._raise("File is not open for writing.")
 
         return write_error
 
     def _is_open(self) -> bool:
-        if self.filedes is not None and self.filedes < len(self._filesystem.open_files):
-            open_files = self._filesystem.open_files[self.filedes]
+        if self.filedes is not None and self.filedes < len(self.filesystem.open_files):
+            open_files = self.filesystem.open_files[self.filedes]
             if open_files is not None and self in open_files:
                 return True
         return False
@@ -1233,13 +1282,13 @@ class FakeFileWrapper:
         if not self.is_stream and not self._is_open():
             raise ValueError("I/O operation on closed file")
 
-    def __iter__(self) -> Union[Iterator[str], Iterator[bytes]]:
-        if not self._read:
+    def __iter__(self) -> Iterator[str] | Iterator[bytes]:
+        if not self.readable():
             self._raise("File is not open for reading")
         return self._io.__iter__()
 
     def __next__(self):
-        if not self._read:
+        if not self.readable():
             self._raise("File is not open for reading")
         return next(self._io)
 
@@ -1249,7 +1298,7 @@ class StandardStreamWrapper:
 
     def __init__(self, stream_object: TextIO):
         self._stream_object = stream_object
-        self.filedes: Optional[int] = None
+        self.filedes: int | None = None
 
     def get_object(self) -> TextIO:
         return self._stream_object
@@ -1258,7 +1307,7 @@ class StandardStreamWrapper:
         """Return the file descriptor of the wrapped standard stream."""
         if self.filedes is not None:
             return self.filedes
-        raise OSError(errno.EBADF, "Invalid file descriptor")
+        raise OSError(errno.EBADF, os.strerror(errno.EBADF))
 
     def read(self, n: int = -1) -> bytes:
         return cast(bytes, self._stream_object.read())
@@ -1270,21 +1319,21 @@ class StandardStreamWrapper:
     def close(self) -> None:
         """We do not support closing standard streams."""
 
-    def close_fd(self, fd: Optional[int]) -> None:
+    def close_fd(self, fd: int | None) -> None:
         """We do not support closing standard streams."""
 
     def is_stream(self) -> bool:
         return True
 
-    def __enter__(self) -> "StandardStreamWrapper":
+    def __enter__(self) -> StandardStreamWrapper:
         """To support usage of this standard stream with the 'with' statement."""
         return self
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         """To support usage of this standard stream with the 'with' statement."""
         self.close()
@@ -1297,12 +1346,12 @@ class FakeDirWrapper:
         self,
         file_object: FakeDirectory,
         file_path: AnyString,
-        filesystem: "FakeFilesystem",
+        filesystem: FakeFilesystem,
     ):
         self.file_object = file_object
         self.file_path = file_path
-        self._filesystem = filesystem
-        self.filedes: Optional[int] = None
+        self._filesystem = weakref.ref(filesystem)
+        self.filedes: int | None = None
 
     def get_object(self) -> FakeDirectory:
         """Return the FakeFile object that is wrapped by the current
@@ -1313,16 +1362,17 @@ class FakeDirWrapper:
         """Return the file descriptor of the file object."""
         if self.filedes is not None:
             return self.filedes
-        raise OSError(errno.EBADF, "Invalid file descriptor")
+        raise OSError(errno.EBADF, os.strerror(errno.EBADF))
 
     def close(self) -> None:
         """Close the directory."""
         self.close_fd(self.filedes)
 
-    def close_fd(self, fd: Optional[int]) -> None:
+    def close_fd(self, fd: int | None) -> None:
         """Close the directory."""
-        assert fd is not None
-        self._filesystem.close_open_file(fd)
+        fs = self._filesystem()
+        assert fs is not None and fd is not None
+        fs.close_open_file(fd)
 
     def read(self, numBytes: int = -1) -> bytes:
         """Read from the directory."""
@@ -1333,15 +1383,15 @@ class FakeDirWrapper:
         self.file_object.write(contents)
         return len(contents)
 
-    def __enter__(self) -> "FakeDirWrapper":
+    def __enter__(self) -> FakeDirWrapper:
         """To support usage of this fake directory with the 'with' statement."""
         return self
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         """To support usage of this fake directory with the 'with' statement."""
         self.close()
@@ -1354,29 +1404,29 @@ class FakePipeWrapper:
 
     def __init__(
         self,
-        filesystem: "FakeFilesystem",
+        filesystem: FakeFilesystem,
         fd: int,
         can_write: bool,
         mode: str = "",
     ):
-        self._filesystem = filesystem
+        self._filesystem = weakref.ref(filesystem)
         self.fd = fd  # the real file descriptor
         self.can_write = can_write
         self.file_object = None
-        self.filedes: Optional[int] = None
+        self.filedes: int | None = None
         self.real_file = None
         if mode:
             self.real_file = open(fd, mode)
 
-    def __enter__(self) -> "FakePipeWrapper":
+    def __enter__(self) -> FakePipeWrapper:
         """To support usage of this fake pipe with the 'with' statement."""
         return self
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         """To support usage of this fake pipe with the 'with' statement."""
         self.close()
@@ -1388,7 +1438,7 @@ class FakePipeWrapper:
         """Return the fake file descriptor of the pipe object."""
         if self.filedes is not None:
             return self.filedes
-        raise OSError(errno.EBADF, "Invalid file descriptor")
+        raise OSError(errno.EBADF, os.strerror(errno.EBADF))
 
     def read(self, numBytes: int = -1) -> bytes:
         """Read from the real pipe."""
@@ -1409,10 +1459,11 @@ class FakePipeWrapper:
         """Close the pipe descriptor."""
         self.close_fd(self.filedes)
 
-    def close_fd(self, fd: Optional[int]) -> None:
+    def close_fd(self, fd: int | None) -> None:
         """Close the pipe descriptor with the given file descriptor."""
-        assert fd is not None
-        open_files = self._filesystem.open_files[fd]
+        fs = self._filesystem()
+        assert fs is not None and fd is not None
+        open_files = fs.open_files[fd]
         assert open_files is not None
         open_files.remove(self)
         if self.real_file:
