@@ -49,10 +49,11 @@ except ImportError:
 
 # From pandas v 1.2 onwards the python fs functions are used even when the engine
 # selected is "c". This means that we don't explicitly have to change the engine.
-patch_pandas = parsers is not None and [int(v) for v in pd.__version__.split(".")] < [
+patch_pandas = parsers is not None and [
+    int(v) for v in pd.__version__.split(".")[:2]
+] < [
     1,
     2,
-    0,
 ]
 
 
