@@ -263,8 +263,10 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         self.os.mkdir(file_dir)
         contents = b"Binary contents"
         with self.open(file_path, "xb") as fake_file:
+            self.assertEqual("xb", fake_file.mode)
             fake_file.write(contents)
         with self.open(file_path, "rb") as fake_file:
+            self.assertEqual("rb", fake_file.mode)
             self.assertEqual(contents, fake_file.read())
 
     def test_overwrite_existing_file(self):
@@ -302,11 +304,28 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         self.assertTrue(self.os.path.exists(file_path))
         with self.open(file_path, "r", encoding="utf8") as fake_file:
             self.assertEqual("old contents", fake_file.read())
+            self.assertEqual("r", fake_file.mode)
         # actual tests
         with self.open(file_path, "w+", encoding="utf8") as fake_file:
             fake_file.write("new contents")
             fake_file.seek(0)
             self.assertTrue("new contents", fake_file.read())
+            self.assertEqual("w+", fake_file.mode)
+
+    def test_open_with_wplus_binary(self):
+        file_path = self.make_path("wplus_file_b")
+        self.create_file(file_path, contents=b"old contents")
+        self.assertTrue(self.os.path.exists(file_path))
+        with self.open(file_path, "rb") as fake_file:
+            self.assertEqual(b"old contents", fake_file.read())
+            self.assertEqual("rb", fake_file.mode)
+        # actual tests
+        with self.open(file_path, "wb+") as fake_file:
+            fake_file.write(b"new contents")
+            fake_file.seek(0)
+            self.assertTrue(b"new contents", fake_file.read())
+            expected_mode = "rb+" if sys.version_info < (3, 15) else "wb+"
+            self.assertEqual(expected_mode, fake_file.mode)
 
     def test_open_with_wplus_truncation(self):
         # set up
@@ -319,6 +338,7 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         with self.open(file_path, "w+", encoding="utf8") as fake_file:
             fake_file.seek(0)
             self.assertEqual("", fake_file.read())
+            self.assertEqual("w+", fake_file.mode)
 
     def test_open_with_append_flag(self):
         contents = [
@@ -335,6 +355,7 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
                 fake_file.read(0)
             with self.assertRaises(io.UnsupportedOperation):
                 fake_file.readline()
+            self.assertEqual("a", fake_file.mode)
             expected_len = len("".join(contents))
             expected_len += len(contents) * (len(self.os.linesep) - 1)
             self.assertEqual(expected_len, fake_file.tell())
@@ -344,6 +365,22 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         with self.open(file_path, encoding="utf8") as fake_file:
             self.assertEqual(contents + additional_contents, fake_file.readlines())
 
+    def test_open_with_append_flag_binary(self):
+        contents = b"Just some boring stuff... "
+        additional_contents = b"some excitement added"
+        file_path = self.make_path("append-binary")
+        self.create_file(file_path, contents=contents)
+        with self.open(file_path, "ab") as fake_file:
+            with self.assertRaises(io.UnsupportedOperation):
+                fake_file.read(0)
+            self.assertEqual("ab", fake_file.mode)
+            self.assertEqual(len(contents), fake_file.tell())
+            fake_file.seek(0)
+            self.assertEqual(0, fake_file.tell())
+            fake_file.write(additional_contents)
+        with self.open(file_path, "rb") as fake_file:
+            self.assertEqual(contents + additional_contents, fake_file.read())
+
     def check_append_with_aplus(self):
         file_path = self.make_path("aplus_file")
         self.create_file(file_path, contents="old contents")
@@ -357,6 +394,7 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
                 self.filesystem, delete_on_close=True
             )
         with self.open(file_path, "a+", encoding="utf8") as fake_file:
+            self.assertEqual("a+", fake_file.mode)
             self.assertEqual(12, fake_file.tell())
             fake_file.write("new contents")
             self.assertEqual(24, fake_file.tell())
@@ -391,6 +429,12 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         with self.open(file_path, "a+", encoding="utf8") as fake_file:
             self.assertEqual("", fake_file.read())
 
+    def test_read_empty_file_with_aplus_binary(self):
+        file_path = self.make_path("aplus_file")
+        with self.open(file_path, "ab+") as fake_file:
+            self.assertEqual(b"", fake_file.read())
+            self.assertEqual("ab+", fake_file.mode)
+
     def test_read_with_rplus(self):
         # set up
         file_path = self.make_path("rplus_file")
@@ -401,11 +445,27 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         # actual tests
         with self.open(file_path, "r+", encoding="utf8") as fake_file:
             self.assertEqual("old contents here", fake_file.read())
+            self.assertEqual("r+", fake_file.mode)
             fake_file.seek(0)
             fake_file.write("new contents")
             fake_file.seek(0)
             self.assertEqual("new contents here", fake_file.read())
 
+    def test_read_with_rplus_binary(self):
+        file_path = self.make_path("rplus_binary")
+        self.create_file(file_path, contents=b"old contents here")
+        self.assertTrue(self.os.path.exists(file_path))
+        with self.open(file_path, "rb") as fake_file:
+            self.assertEqual(b"old contents here", fake_file.read())
+
+        with self.open(file_path, "rb+") as fake_file:
+            self.assertEqual(b"old contents here", fake_file.read())
+            self.assertEqual("rb+", fake_file.mode)
+            fake_file.seek(0)
+            fake_file.write(b"new contents")
+            fake_file.seek(0)
+            self.assertEqual(b"new contents here", fake_file.read())
+
     def create_with_permission(self, file_path, perm_bits):
         self.create_file(file_path)
         self.os.chmod(file_path, perm_bits)
@@ -900,6 +960,7 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         file_obj = self.filesystem.get_object(filename)
         with self.open(fd, "wb", closefd=False) as fp:
             fp.write(b"test")
+            self.assertEqual("wb", fp.mode)
         self.assertTrue(self.filesystem.has_open_file(file_obj))
         self.os.close(fd)
         self.assertFalse(self.filesystem.has_open_file(file_obj))
@@ -947,6 +1008,29 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
             text = f.read()
             assert text == "12"
 
+    def test_readable_writable(self):
+        # regression test for #1265
+        file_path = self.make_path("foo")
+        self.create_file(file_path, contents="test")
+        with self.open(file_path, "r") as f:
+            self.assertTrue(f.readable())
+            self.assertFalse(f.writable())
+        with self.open(file_path, "w") as f:
+            self.assertFalse(f.readable())
+            self.assertTrue(f.writable())
+        with self.open(file_path, "a") as f:
+            self.assertFalse(f.readable())
+            self.assertTrue(f.writable())
+        with self.open(file_path, "r+") as f:
+            self.assertTrue(f.readable())
+            self.assertTrue(f.writable())
+        with self.open(file_path, "w+") as f:
+            self.assertTrue(f.readable())
+            self.assertTrue(f.writable())
+        with self.open(file_path, "a+") as f:
+            self.assertTrue(f.readable())
+            self.assertTrue(f.writable())
+
 
 class RealFileOpenTest(FakeFileOpenTest):
     def use_real_fs(self):
@@ -1028,6 +1112,7 @@ class FakeFileOpenWithOpenerTest(FakeFileOpenTestBase):
 
         file_path = self.make_path("bar")
         with self.open(file_path, "x", encoding="utf8", opener=self.opener) as f:
+            self.assertEqual("x", f.mode)
             assert f.write("bar") == 3
             with self.assertRaises(OSError):
                 f.read()
@@ -1042,6 +1127,7 @@ class FakeFileOpenWithOpenerTest(FakeFileOpenTestBase):
 
         file_path = self.make_path("bar")
         with self.open(file_path, "x+", encoding="utf8", opener=self.opener) as f:
+            self.assertEqual("x+", f.mode)
             assert f.write("bar") == 3
             assert f.read() == ""
         with self.open(file_path, encoding="utf8") as f:
@@ -1053,7 +1139,6 @@ class RealFileOpenWithOpenerTest(FakeFileOpenWithOpenerTest):
         return True
 
 
-@unittest.skipIf(sys.version_info < (3, 8), "open_code only present since Python 3.8")
 class FakeFilePatchedOpenCodeTest(FakeFileOpenTestBase):
     def setUp(self):
         super().setUp()
@@ -1101,7 +1186,6 @@ class RealPatchedFileOpenCodeTest(FakeFilePatchedOpenCodeTest):
         return True
 
 
-@unittest.skipIf(sys.version_info < (3, 8), "open_code only present since Python 3.8")
 class FakeFileUnpatchedOpenCodeTest(FakeFileOpenTestBase):
     def setUp(self):
         super().setUp()
@@ -1227,26 +1311,26 @@ class BufferingModeTest(FakeFileOpenTestBase):
     def test_writing_large_text_with_line_buffer(self):
         file_path = self.make_path("buffertest.bin")
         with self.open(file_path, "w", encoding="utf8", buffering=1) as f:
-            f.write("test" * 4000)
+            f.write("test" * 33000)
             with self.open(file_path, "r", encoding="utf8") as r:
                 x = r.read()
                 # buffer larger than default - written
-                self.assertEqual(16000, len(x))
+                self.assertEqual(132000, len(x))
             f.write("test")
             with self.open(file_path, "r", encoding="utf8") as r:
                 x = r.read()
                 # buffer not filled - not written
-                self.assertEqual(16000, len(x))
+                self.assertEqual(132000, len(x))
             f.write("\ntest")
             with self.open(file_path, "r", encoding="utf8") as r:
                 x = r.read()
                 # new line - buffer written
-                self.assertEqual(16009, len(x))
+                self.assertEqual(132009, len(x))
             f.write("\ntest")
             with self.open(file_path, "r", encoding="utf8") as r:
                 x = r.read()
                 # another new line - buffer written
-                self.assertEqual(16014, len(x))
+                self.assertEqual(132014, len(x))
 
     def test_writing_text_with_default_buffer(self):
         file_path = self.make_path("buffertest.txt")
@@ -1374,11 +1458,11 @@ class BufferingModeTest(FakeFileOpenTestBase):
         line_end_size = len(self.os.linesep)
         char_count = io.DEFAULT_BUFFER_SIZE // 256 - line_end_size
         for line_count in (255, 256, 257, 511, 512, 513):
-            with self.open(file_path, "w") as f:
+            with self.open(file_path, "w", encoding="utf8") as f:
                 for i in range(line_count):
                     f.write("x" * char_count + "\n")
 
-            with self.open(file_path) as f:
+            with self.open(file_path, encoding="utf8") as f:
                 lines = f.readlines()
                 self.assertEqual(line_count, len(lines))
 
